#!/usr/bin/env python
"""

Build common block mechanism for f2py2e.

Copyright 2000 Pearu Peterson all rights reserved,
Pearu Peterson <pearu@ioc.ee>          
Permission to use, modify, and distribute this software is given under the
terms of the LGPL.  See http://www.fsf.org

NO WARRANTY IS EXPRESSED OR IMPLIED.  USE AT YOUR OWN RISK.
$Date: 2000/03/16 11:07:31 $
Pearu Peterson
"""

__version__ = "$Revision: 1.6 $"[10:-1]

f2py_version='See `f2py -v`'

import pprint
import sys,string,time,types,copy
errmess=sys.stderr.write
outmess=sys.stdout.write
show=pprint.pprint

from auxfuncs import *
import capi_maps
import cfuncs
##############

common_rules={
    'body':"""
#begintitle#
#ifndef COMMONBLOCKWRAPPER#NAME#
#define COMMONBLOCKWRAPPER#NAME#
static char doc_#apiname#[] = \"\\\nCommon signature:\\n\\
\t{#keys#} = get_#name#()\\n\\
Keys:\\n\\
#docstr#\";
extern struct {
#commondecl#
} F_FUNC(#name#,#NAME#); 
static PyObject *#apiname#(PyObject *capi_self, PyObject *capi_args) {
#decl#
\tif (!PyArg_ParseTuple(capi_args, \"\")) goto capi_fail;
\treturn Py_BuildValue(\"{#keyarrformat#}\"#keyarr#);
capi_fail:
\treturn NULL;
}
#endif
#endtitle#
    """,
    'method':'#ifndef COMMONBLOCKWRAPPER#NAME#METHOD\n#define COMMONBLOCKWRAPPER#NAME#METHOD\n\t{\"get_#name#\",#apiname#,METH_VARARGS,doc_#apiname#},\n#endif',
    'need':['CFUNCSMESS'],
    'latexdoc':"""
\\providecommand{\\commonblock#texnamename#}{\\newif\\ifcb#texnamename#\\cb#texnamename#true}
\\commonblock#texnamename#
\\ifcb#texnamename#
\\subsection{Common block wrapper \\texttt{get\_#texname#}}
                                           
\\noindent{{}\\verb@{#keys#} = get_#name#()@{}}
#ssecnote#

\\begin{description}
#latexdocstr#
\\end{description}
\\cb#texnamename#false
\\fi
\\renewcommand{\\commonblock#texnamename#}{}
"""
    }
com_rules=[
    {'decl':'/* decl */','latexdocstr':'%%',
     'commondecl':'/* commondecl */',
     'keyarr':'','keys':[]},
    {'ssecnote':{hasnote:'--- #note#',l_not(hasnote):''}}
    ]
var_rules=[
    {'separatorsfor':{'commondecl':'\n','keyarr':'\\\n\t\t',
                      'keyarrformat':',','decl':'\n','keys':',',
                      'docstr':'\\n\\\n','latexdocstr':'\n'
                      },
     },{
     'keys':'\'#varname#\'',
     'docstr':'\t#arrdocstr#',
     'latexdocstr':['\\item[]{{}\\verb@#arrdocstr#@{}}',
                    {hasnote:'--- #note#'}],
     '_check':isintent_nothide
     },
    {'_check':isscalar,
     'commondecl':'\t#ctype# #varname#;',
     'keyarr':{isintent_nothide:',"#varname#",PyArray_FromDimsAndData(0,NULL,#atype#,(char *)(&(F_FUNC(#name#,#NAME#).#varname#)))'},
     'keyarrformat':{isintent_nothide:'s:N'},
     },
    {'_check':isarray,
     'commondecl':'\t#ctype# #varname#[#size#];',
     'decl':{isintent_nothide:'\tint #varname#_Dims[] = {#dims#};'},
     'keyarr':{isintent_nothide:',"#varname#",PyArray_FromDimsAndData(#rank#,#varname#_Dims,#atype#,(char *)(F_FUNC(#name#,#NAME#).#varname#))'},
     'keyarrformat':{isintent_nothide:'s:N'},
     },
    {'_check':l_and(isstring,l_not(isarray)),
     'commondecl':'\tchar #varname#[#size#];',
     'decl':{isintent_nothide:'\tint #varname#_Dims[] = {#size#};'},
     'keyarr':{isintent_nothide:',"#varname#",PyArray_FromDimsAndData(#rank#,#varname#_Dims,PyArray_CHAR,(char *)(F_FUNC(#name#,#NAME#).#varname#))'},
     'keyarrformat':{isintent_nothide:'s:N'},
     },
    ]

###############

def buildcommonblocks(rout):
    if not hascommon(rout): return {}
    ret={}
    for k in rout['common'].keys():
        ret=dictappend(ret,buildcommonblock(k,rout))
    return ret
def buildcommonblock(name,rout):
    outmess('\t\tConstructing common block "%s" wrapper\n'%(name))
    vrd={'name':name,'NAME':string.upper(name),
        'texname':string.replace(name,'_','\\_'),
        'begintitle':gentitle(name),
        'endtitle':gentitle('end of %s'%name),
        'apiname':'#modulename#_common_%s_in_%s'%(name,rout['name'])
        }
    nummap={0:'Ro',1:'Ri',2:'Rii',3:'Riii',4:'Riv',5:'Rv',6:'Rvi',7:'Rvii',8:'Rviii',9:'Rix'}
    vrd['texnamename']=name
    for i in nummap.keys():
        vrd['texnamename']=string.replace(vrd['texnamename'],`i`,nummap[i])
    if hasnote(rout): vrd['note']=rout['note']
    rd=dictappend({},vrd)
    var=rout['vars']
    for r in com_rules:
        if (r.has_key('_check') and r['_check'](rout)) or (not r.has_key('_check')):
            ar=applyrules(r,vrd)
            rd=dictappend(rd,ar)
    for a in rout['common'][name]:
        vrd=capi_maps.common_sign2map(a,var[a])
        if not l_or(isstring,isscalar,isarray)(var[a]):
            errmess('buildcommonblock: only string,scalar,array are supported: set_%s will be broken\n'%name)
        for r in var_rules:
            if (r.has_key('_check') and r['_check'](var[a])) or (not r.has_key('_check')):
                ar=applyrules(r,vrd,var[a])
                rd=dictappend(rd,ar)
    ar=applyrules(common_rules,rd)
    #print ar['body']
    return ar



