#!/usr/bin/env python
"""

f2py2e - Fortran to Python C/API generator. 2nd Edition.
         See __usage__ below.

Copyright 1999,2000 Pearu Peterson all rights reserved,
Pearu Peterson <pearu@ioc.ee>          
Permission to use, modify, and distribute this software is given under the
terms of the LGPL.  See http://www.fsf.org

NO WARRANTY IS EXPRESSED OR IMPLIED.  USE AT YOUR OWN RISK.
$Date: 2000/09/12 11:39:13 $
Pearu Peterson
"""
__version__ = "$Revision: 1.26 $"[10:-1]

import sys,os,string,pprint
errmess=sys.stderr.write
outmess=sys.stdout.write
show=pprint.pprint

import crackfortran
import rules
import cb_rules
import common_rules
import auxfuncs
import cfuncs
import capi_maps
import buildmakefile
import func2subr
import f90mod_rules

f2py_version=-1.263 # The version of the last release

if f2py_version<=0: # Calculate the version
    v=__version__
    f2py_version=eval(v[string.find(v,'.')+1:])
    for m in [crackfortran,rules,cb_rules,common_rules,cfuncs,auxfuncs,capi_maps,
              buildmakefile,func2subr,f90mod_rules]:
        v=m.__version__
        f2py_version=f2py_version+eval(v[string.find(v,'.')+1:])
f2py_version='2.%d'%f2py_version
#             ^     ^__________^___ this is cumulative revision number 
#             \___ this is public release number

for m in [crackfortran,rules,cb_rules,common_rules,cfuncs,auxfuncs,capi_maps,buildmakefile]:
    m.f2py_version = f2py_version

__usage__ = """\
Usage:
      f2py [<options>] <fortran files> [[[only:]||[skip:]] <fortran functions> ]\\
                                       [: <fortran files> ...]
Description: This program generates a Python C/API file (<modulename>module.c)
             that wraps given fortran functions so that they can be called
             from Python.
Options:
  -f77             Use if fortran code is in Fortran 77 fixed format. Default.
  -f90             Use if fortran code is in Fortran 90/95 free format.
  -fix             Use if fortran code is in Fortran 90/95 fixed format.
  -h <filename>    Write signatures of the fortran routines to file <filename>
                   and exit. You can then edit <filename> and use it instead
                   of <fortran files>.
  -m <modulename>  Name of the module; f2py2e generates a file <modulename>module.c
  <fortran functions>  Names of fortran routines for which Python C/API functions will
                   be generated. Default is all that are found in <fortran files>.
  <fortran files>  Paths to fortran/signature files that will be scanned for <fortran functions>
                   in order to determine their signatures.
  skip:            Ignore fortran functions that follow.
  only:            Use only fortran functions that follow.
  :                Get back to <fortran files> mode.
  --[no]-lower     Do [not] lower the cases in <fortran files>. By default,
                   --lower is assumed with -h key, and --no-lower without -h key.
  --short-latex    Create 'incomplete' LaTeX document (without commands \\documentclass,
                   \\tableofcontents, and \\begin{document},\\end{document}).
  --debug-capi     Create C/API code that will be very verbose. Use when debugging.
  --overwrite-signature  Overwrite existing signature file.
  --overwrite-makefile  Overwrite existing `Makefile-<modulename>' file.
  -v               Print version ID of f2py and exit.
  -pyinc           Print Python include path and exit (was -I).
  -include'<includefile>' Add CPP #include statement to the C/API code. <includefile> should
                   be in the format of either `\"filename.ext\"' or `<filename.ext>'.
                   As a result <includefile> will be included just before wrapper functions in
                   the C/API code.
  -makefile <options>  Give additional information for building file `Makefile-<modulename>'.
                   The following <options> are used:
                   `-L<libpath>', `-l<libname>', `-I<incpath>',`[-D|-U]<cpp-macro>',
                   `<additional files *.{f,f90,..} *.c that need to be compiled>',
                   and makefile variables such as F2PYFLAGS,
                   CC,CFLAGS,COPT,FC,FFLAGS,LDSHARED,LDFLAGS,etc (they'll overwrite
                   default ones found in the environment; see buildmakefile.py).
                   The scope of `-makefile' options ends with `:'.
                   By default all <fortran files> will be included to the list of files
                   that need to be compiled. To disable this feature use `--use-libs' option.
  --[no]-wrap-functions Create Fortran subroutine wrappers for Fortran functions (default).
Version:     %s
Requires:    Python 1.5.2 or higher
             Numerical Python 13 or higher
Copyright 1999, 2000 Pearu Peterson all rights reserved.
WWW page: http://cens.ioc.ee/projects/f2py2e/"""%(f2py_version)


def scaninputline(inputline):
    files,funcs,skipfuncs,onlyfuncs,debug=[],[],[],[],[]
    f,f2,f3,f4,f5=1,0,0,0,0
    dolc=-1
    wrapfuncs = 1
    signsfile,modulename=None,None
    format='f77'
    makefileopts=[]
    options={}
    for l in inputline:
        if l=='': pass
        elif l=='only:': f=0
        elif l=='skip:': f=-1
        elif l==':': f=1;f4=0
        elif l=='-f77': format='f77'
        elif l=='-f90': format='f90'
        elif l=='-fix': format='fix'
        elif l[:8]=='--debug-': debug.append(l[8:])
        elif l=='--lower': dolc=1
        elif l=='--no-lower': dolc=0
        elif l=='--wrap-functions': wrapfuncs=1
        elif l=='--no-wrap-functions': wrapfuncs=0
        elif l=='--short-latex': options['shortlatex']=1
        elif l=='--overwrite-signature': options['h-overwrite']=1
        elif l=='--overwrite-makefile': makefileopts.append('--overwrite-makefile')
        elif l=='-h': f2=1
        elif l=='-m': f3=1
        elif l=='--use-libs':
            makefileopts.append(l)
        elif l=='-makefile':
            f4=1
        elif l[:2]=='-v':
            print f2py_version
            sys.exit()
        elif l[:6]=='-pyinc':
            print sys.prefix+'/include/python'+sys.version[:3]
            sys.exit()
        elif l=='--show-compilers':
            f5=1
        elif l[:8]=='-include':
            cfuncs.outneeds['userincludes'].append(l[9:-1])
            cfuncs.userincludes[l[9:-1]]='#include '+l[8:]
        elif l[0]=='-':
            if f4:
                makefileopts.append(l)
            else:
                errmess('Unknown option %s\n'%`l`)
        elif f2: f2=0;signsfile=l
        elif f3: f3=0;modulename=l
        elif f4:
            makefileopts.append(l)
        elif f==1:
            try:
                open(l).close()
                files.append(l)
            except IOError,detail:
                errmess('IOError: %s. Skipping file "%s".\n'%(str(detail),l))
        elif f==-1: skipfuncs.append(l)
        elif f==0: onlyfuncs.append(l)
    if not f5 and not files and not modulename:
        print __usage__
        sys.exit()
    if signsfile and os.path.isfile(signsfile) and not options.has_key('h-overwrite'):
        errmess('Signature file "%s" exists!!! Use --overwrite-signature to overwrite.\n'%(signsfile))
        sys.exit()
    if format in ['f77','f90','fix']: options['format']=format
    else:
        errmess('Illegal format "%s". Exiting.\n'%(format))
        sys.exit()
    options['debug']=debug
    if dolc==-1 and not signsfile: options['do-lower']=0
    else: options['do-lower']=dolc
    if modulename: options['module']=modulename
    if signsfile: options['signsfile']=signsfile
    if onlyfuncs: options['onlyfuncs']=onlyfuncs
    if skipfuncs: options['skipfuncs']=skipfuncs
    options['wrapfuncs'] = wrapfuncs
    if not '--use-libs' in makefileopts:
        makefileopts=makefileopts+files
    options['makefileopts']=makefileopts
    if f5:
        buildmakefile.showcompilers(options)
        sys.exit()
    return files,options
def callcrackfortran(files,options):
    rules.options=options
    funcs=[]
    if options['format']=='f77':
        crackfortran.strictf77=1
        crackfortran.sourcecodeform='fix'
    elif options['format']=='f90':
        crackfortran.strictf77=0
        crackfortran.sourcecodeform='free'
    elif options['format']=='fix':
        crackfortran.strictf77=0
        crackfortran.sourcecodeform='fix'
    else:
        errmess('callcrackfortran: Unknown format. Exiting.\n');sys.exit()
    crackfortran.debug=options['debug']
    if options.has_key('module'):
        crackfortran.f77modulename=options['module']
    if options.has_key('skipfuncs'):
        crackfortran.skipfuncs=options['skipfuncs']
    if options.has_key('onlyfuncs'):
        crackfortran.onlyfuncs=options['onlyfuncs']
    crackfortran.dolowercase=options['do-lower']
    postlist=crackfortran.crackfortran(files)
    if options.has_key('signsfile'):
        outmess('Saving signatures to file "%s"\n'%(options['signsfile']))
        pyf=crackfortran.crack2fortran(postlist)
        f=open(options['signsfile'],'w')
        f.write(pyf)
        f.close()
    return postlist
def buildmodules(list):
    cfuncs.buildcfuncs()
    outmess('Building modules...\n')
    modules,mnames,isusedby=[],[],{}
    for i in range(len(list)):
        if string.find(list[i]['name'],'__user__')>=0:
            cb_rules.buildcallbacks(list[i])
        else:
            if list[i].has_key('use'):
                for u in list[i]['use'].keys():
                    if not isusedby.has_key(u): isusedby[u]=[]
                    isusedby[u].append(list[i]['name'])
            modules.append(list[i])
            mnames.append(list[i]['name'])
    for i in range(len(mnames)):
        if isusedby.has_key(mnames[i]):
            outmess('\tSkipping module "%s" which is used by %s.\n'%(mnames[i],string.join(map(lambda s:'"%s"'%s,isusedby[mnames[i]]),',')))
        else:
            um=[]
            if modules[i].has_key('use'):
                for u in modules[i]['use'].keys():
                    if isusedby.has_key(u) and u in mnames:
                        um.append(modules[mnames.index(u)])
                    else:
                        outmess('\tModule "%s" uses nonexisting "%s" which will be ignored.\n'%(mnames[i],u))
            rules.buildmodule(modules[i],um)

if __name__ == "__main__":
    files,options=scaninputline(sys.argv[1:])
    postlist=callcrackfortran(files,options)
    isusedby={}
    for i in range(len(postlist)):
        if postlist[i].has_key('use'):
            for u in postlist[i]['use'].keys():
                if not isusedby.has_key(u): isusedby[u]=[]
                isusedby[u].append(postlist[i]['name'])
    for i in range(len(postlist)):
        if postlist[i]['block']=='module' and string.find(postlist[i]['name'],'__user__')<0:
            if isusedby.has_key(postlist[i]['name']):
                outmess('Skipping Makefile build for module "%s" which is used by %s\n'%(postlist[i]['name'],string.join(map(lambda s:'"%s"'%s,isusedby[postlist[i]['name']]),',')))
            else:
                buildmakefile.build(postlist[i],options)
    if options.has_key('signsfile'):
        outmess('Stopping. Edit the signature file and then run f2py on the signature file: ')
        outmess('%s %s\n'%(os.path.basename(sys.argv[0]),options['signsfile']))
        outmess('\tOr run GNU make to build shared module: gmake -f Makefile-<modulename>\n')
        sys.exit()
    for i in range(len(postlist)):
        if postlist[i]['block']!='module':
            errmess('All blocks must be module blocks but got %s. Exiting.\n'%(`postlist[i]['block']`))
            if not options.has_key('module'):
                errmess('Tip: If your original code is Fortran 77 then you must use -m option.\n')
            sys.exit()
    auxfuncs.debugoptions=options['debug']
    auxfuncs.wrapfuncs=options['wrapfuncs']
    buildmodules(postlist)
    outmess('Run GNU make to build shared modules: gmake -f Makefile-<modulename> [test]\n')
# EOF
