#!/usr/bin/env python
"""

Build F90 module support for f2py2e.

Copyright 2000 Pearu Peterson all rights reserved,
Pearu Peterson <pearu@ioc.ee>          
Permission to use, modify, and distribute this software is given under the
terms of the LGPL.  See http://www.fsf.org

NO WARRANTY IS EXPRESSED OR IMPLIED.  USE AT YOUR OWN RISK.
$Date: 2000/09/11 20:38:11 $
Pearu Peterson
"""

__version__ = "$Revision: 1.1 $"[10:-1]

f2py_version='See `f2py -v`'

import pprint
import sys,string,time,types,copy
errmess=sys.stderr.write
outmess=sys.stdout.write
show=pprint.pprint

from auxfuncs import *
import capi_maps
import cfuncs

def buildhooks(m):
    outmess('\t\tConstructing F90 module support for "%s"...'%(m['name']))
    ret = {'need':[],'initf90modhooksstatic':'',
           'initf90modhooksdynamic':''}
    staticargs,dynamicargs = [],[]
    staticargsc,dynamicargsc = [],[]
    for rout in m['body']: # module functions are treated in func2subr.py
        if issubroutine(rout) and rout['name'] not in m['interfaced']:
            staticargs.append(rout['name'])
            staticargsc.append('void (*%s)()'%(rout['name']))
    if m.has_key('vars'):
        for v in m['vars'].keys():
            ct = capi_maps.getctype(m['vars'][v])
            if cfuncs.typedefs.has_key(ct):
                ret['need'].append(ct)
            if isallocatable(m['vars'][v]):
                dynamicargs.append(v)
                dynamicargsc.append('%s *%s'%(ct,v))
            else:
                staticargs.append(v)
                staticargsc.append('%s *%s'%(ct,v))

    if not staticargs+dynamicargs:
        outmess('None\n')
        return ret,''
    outmess('\n\t\t\t%s\n'%(string.join(staticargs+dynamicargs,' ')))
    modsetup = ''
    fwrap = ['']
    def fadd(line,fwrap=fwrap): fwrap[0] = '%s\n      %s'%(fwrap[0],line)
    if staticargs:
        fadd('subroutine f2pymodwrapstatic%s (f2pysetupfunc)'%(m['name']))
        fadd('use %s'%(m['name']))
        #fadd('write(*,*) "Calling f2pysetupfunc"')
        fadd('call f2pysetupfunc(%s)'%(string.join(staticargs,',')))
        #fadd('write(*,*) "Returned from f2pysetupfunc"')
        fadd('end subroutine')
        modsetup = modsetup + """
extern void F_FUNC(f2pymodwrapstatic#modulename#,F2PYMODWRAPSTATIC#MODULENAME#)();
static void f2pymodsetupstatic#modulename# (%s) {
  %s;
}
"""%(string.join(staticargsc,','),
     string.join(map(lambda a:'f2pymodstruct#modulename#.%s = %s'%(a,a),staticargs),';\n  '))
    if dynamicargs:
        fadd('subroutine f2pymodwrapdynamic%s (f2pysetupfunc)'%(m['name']))
        fadd('use %s'%(m['name']))
        fadd('call f2pysetupfunc(%s)'%(string.join(dynamicargs,',')))
        fadd('end subroutine')
        modsetup = modsetup + """
extern void F_FUNC(f2pymodwrapdynamic#modulename#,F2PYMODWRAPDYNAMIC#MODULENAME#)();
static void f2pymodsetupdynamic#modulename# (%s) {
  %s;
}
"""%(string.join(dynamicargsc,','),
     string.join(map(lambda a:'f2pymodstruct#modulename#.%s = %s'%(a,a),dynamicargs),';\n  '))

    modstruct = """
static struct {
  %s;
} f2pymodstruct#modulename#;
"""%(string.join(staticargsc+dynamicargsc,';\n  '))

    
    ret['f90modhooks'] = modstruct + modsetup
    ret['initf90modhooksstatic'] = 'F_FUNC(f2pymodwrapstatic#modulename#,F2PYMODWRAPSTATIC#MODULENAME#)(f2pymodsetupstatic#modulename#);'
    ret['initf90modhooksdynamic'] = 'F_FUNC(f2pymodwrapdynamic#modulename#,F2PYMODWRAPDYNAMIC#MODULENAME#)(f2pymodsetupdynamic#modulename#);'
    ret['need'].append('F_FUNC')
    return ret,fwrap[0]
