#!/usr/bin/env python
"""

Rules for building C/API module with f2py2e.

Copyright 1999,2000 Pearu Peterson all rights reserved,
Pearu Peterson <pearu@ioc.ee>          
Permission to use, modify, and distribute this software is given under the
terms of the LGPL.  See http://www.fsf.org

NO WARRANTY IS EXPRESSED OR IMPLIED.  USE AT YOUR OWN RISK.
$Date: 2000/09/11 20:34:42 $
Pearu Peterson
"""

__version__ = "$Revision: 1.42 $"[10:-1]

f2py_version='See `f2py -v`'

import pprint
import sys,string,time,types,copy
errmess=sys.stderr.write
outmess=sys.stdout.write
show=pprint.pprint

from auxfuncs import *
import capi_maps
from capi_maps import *
import cfuncs
import common_rules
import use_rules
import f90mod_rules
import func2subr
options={}

sepdict={}
#for k in ['need_cfuncs']: sepdict[k]=','
for k in ['decl','frompyobj','topyarr','method','pyobjfrom','freemem','userincludes',
          'includes0','includes','typedefs','cppmacros','cfuncs','callbacks',
          'latexdoc']:
    sepdict[k]='\n'

#################### Rules for C/API module #################

module_rules={
    'modulebody':"""\
/* File: #modulename#module.c
 * This file is auto-generated with f2py (version:#f2py_version#).
 * f2py is a Fortran to Python Interface Generator (FPIG), Second Edition,
 * written by Pearu Peterson <pearu@ioc.ee>.
 * See http://cens.ioc.ee/projects/f2py2e/
 * Generation date: """+time.asctime(time.localtime(time.time()))+"""
 * $R"""+"""evision:$
 * $D"""+"""ate:$
 * Do not edit this file directly unless you know what you are doing!!!
 */
#ifdef __CPLUSPLUS__
extern \"C\" {
#endif

"""+gentitle("See f2py2e/cfuncs.py: includes")+"""
#includes#
#includes0#

"""+gentitle("See f2py2e/rules.py: mod_rules['modulebody']")+"""
static PyObject *#modulename#_error;
static PyObject *#modulename#_control;

"""+gentitle("See f2py2e/cfuncs.py: typedefs")+"""
#typedefs#

"""+gentitle("See f2py2e/cfuncs.py: cppmacros")+"""
#cppmacros#

"""+gentitle("See f2py2e/cfuncs.py: cfuncs")+"""
#cfuncs#

"""+gentitle("See f2py2e/cb_rules.py: buildcallback")+"""
#callbacks#

"""+gentitle("See f2py2e/f90mod_rules.py: buildhooks")+"""
#f90modhooks#

"""+gentitle("See f2py2e/cfuncs.py: userincludes")+"""
#userincludes#

"""+gentitle("See f2py2e/rules.py: buildapi")+"""
#body#

"""+gentitle("See f2py2e/rules.py: module_rules['modulebody']")+"""

static PyMethodDef #modulename#_module_methods[] = {
#method#
\t{NULL,NULL}
};
void init#modulename#() {
\tPyObject *m, *d, *s;
\tm = Py_InitModule(\"#modulename#\", #modulename#_module_methods);
\timport_array();
\t#modulename#_control = PyDict_New();
\td = PyModule_GetDict(m);
\ts = PyString_FromString(\"$R"""+"""evision: $\");
\tPyDict_SetItemString(d, \"__version__\", s);
\ts = PyString_FromString(\"This module '#modulename#' is auto-generated with f2py (version:#f2py_version#).\\nThe following functions are available:\\n#docs#.\");
\tPyDict_SetItemString(d, \"__doc__\", s);
\tPyDict_SetItemString(d, \"_control\", #modulename#_control);
\t#modulename#_error = PyErr_NewException (\"#modulename#.error\", NULL, NULL);
\tPy_DECREF(s);
\t#initf90modhooksstatic#
\tif (PyErr_Occurred())
\t\tPy_FatalError(\"can't initialize module #modulename#\");
}
#ifdef __CPLUSCPLUS__
}
#endif
""",
    'separatorsfor':{'latexdoc':'\n\n'},
    'latexdoc':['\\section{Module \\texttt{#texmodulename#}}\n',
                '#modnote#\n',
                '#latexdoc#']
    }

defmod_rules=[
    {'body':'/*eof body*/',
     'method':'/*eof method*/',
     'latexdoc':'',
     'modnote':{hasnote:'#note#',l_not(hasnote):''},
     }
    ]

routine_rules={
    'separatorsfor':sepdict,
    'body':"""
#begintitle#
static char doc_#apiname#[] = \"\\\nFunction signature:\\n\\\n\t#docreturn##name#(#docsignatureshort#)\\n\\\n#docstrsigns#\";
#declfortranroutine#
static PyObject *#apiname#(PyObject *capi_self, PyObject *capi_args, PyObject *capi_keywds) {
\tPyObject *capi_buildvalue = NULL;
#decl#
\tstatic char *capi_kwlist[] = {#kwlist##kwlistopt##kwlistxa#NULL};
#routdebugenter#
\tif (!PyArg_ParseTupleAndKeywords(capi_args,capi_keywds,\\
\t\t\"#argformat#|#keyformat##xaformat#:#pyname#\",\\
\t\tcapi_kwlist#args_capi##keys_capi##keys_xa#))\n\t\tgoto capi_fail;
#frompyobj#
#callfortranroutine#
#pyobjfrom#
\tCFUNCSMESS(\"Building return value.\\n\");
\tcapi_buildvalue = Py_BuildValue(\"#returnformat#\"#return#);
capi_fail:
\tif (capi_buildvalue == NULL) {
#routdebugfailure#
\t} else {
#routdebugleave#
\t}
\tCFUNCSMESS(\"Freeing memory.\\n\");
#freemem#
\treturn capi_buildvalue;
}
#endtitle#
""",
    'method':'\t{\"#name#\",(PyCFunction)#apiname#,METH_VARARGS|METH_KEYWORDS,doc_#apiname#},',
    'doc':'#docreturn##name#(#docsignature#)',
    'docshort':'#docreturn##name#(#docsignatureshort#)',
    'docs':'\t#docreturn##name#(#docsignature#)\\n',
    'need':['arrayobject.h','CFUNCSMESS'],
    'cppmacros':{debugcapi:'#define DEBUGCFUNCS'},
    'latexdoc':['\\subsection{Wrapper function \\texttt{#texname#}}\n',
                """
\\noindent{{}\\verb@#docreturn##name#@{}}\\texttt{(#latexdocsignatureshort#)}
#routnote#

#latexdocstrsigns#
"""]
    }

################## Rules for C/API function ##############

rout_rules=[
    { # Init
    'separatorsfor': {'callfortranroutine':'\n','routdebugenter':'\n','decl':'\n',
                      'routdebugleave':'\n','routdebugfailure':'\n',
                      'setjmpbuf':' || ',
                      'docstrreq':'\n','docstropt':'\n','docstrout':'\n',
                      'docstrcbs':'\n','docstrsigns':'\\n\\\n',
                      'latexdocstrsigns':'\n',
                      'latexdocstrreq':'\n','latexdocstropt':'\n',
                      'latexdocstrout':'\n','latexdocstrcbs':'\n',
                      },
    'kwlist':'','kwlistopt':'','callfortran':'','callfortranappend':'',
    'docsign':'','docsignopt':'','decl':'/*decl*/','freemem':'/*freemem*/',
    'docsignshort':'','docsignoptshort':'',
    'docstrsigns':'','latexdocstrsigns':'',
    'docstrreq':'Required arguments:',
    'docstropt':'Optional arguments:',
    'docstrout':'Return objects:',
    'docstrcbs':'Call-back functions:',
    'latexdocstrreq':'\\noindent Required arguments:',
    'latexdocstropt':'\\noindent Optional arguments:',
    'latexdocstrout':'\\noindent Return objects:',
    'latexdocstrcbs':'\\noindent Call-back functions:',
    'args_capi':'','keys_capi':'',
    'frompyobj':'/*frompyobj*/','pyobjfrom':'/*pyobjfrom*/',
    'topyarr':'/*topyarr*/','routdebugleave':'/*routdebugleave*/',
    'routdebugenter':'/*routdebugenter*/',
    'routdebugfailure':'/*routdebugfailure*/',
    'callfortranroutine':'/*callfortranroutine*/',
    'argformat':'','keyformat':'','need_cfuncs':'',
    'docreturn':'','return':'','returnformat':'','rformat':'',
    'kwlistxa':'','keys_xa':'','xaformat':'','docsignxa':'','docsignxashort':'',
    'routnote':{hasnote:'--- #note#',l_not(hasnote):''},
    },{
    'apiname':'#modulename#_#name#',
    'pyname':'#modulename#.#name#',
    'decl':''
    },{ # Subroutine
    'declfortranroutine':{l_not(ismoduleroutine):'extern void F_FUNC(#name#,#NAME#)();',
                          ismoduleroutine:''},
    'need':{l_not(ismoduleroutine):'F_FUNC',ismoduleroutine:'F_MODFUNC'},
    'callfortranroutine':[
        {debugcapi:["""\tfprintf(stderr,\"debug-capi:Fortran subroutine `#name#(#callfortran#)\'\\n\");"""]},
        {hasexternals:"""\
\tif (#setjmpbuf#) {
\t\tif ((PyErr_Occurred())==NULL)
\t\t\tPyErr_SetString(#modulename#_error,\"Failure of a callback function\");
\t\tgoto capi_fail;
\t} else"""},
        {l_not(ismoduleroutine):'\tF_FUNC(#name#,#NAME#)(#callfortran#);',
         ismoduleroutine:'\tF_MODFUNC(#modulename#,#name#)(#callfortran#);',
         }],
    '_check':issubroutine,
    },{ # Wrapped function
    'declfortranroutine':'extern void F_WRAPPEDFUNC(#name#,#NAME#)();',
    'need':'F_WRAPPEDFUNC',
    'callfortranroutine':[
    {debugcapi:["""\tfprintf(stderr,\"debug-capi:Fortran subroutine `f2pywrap#name#(#callfortran#)\'\\n\");"""]},
    {hasexternals:"""\
\tif (#setjmpbuf#) {
\t\tif ((PyErr_Occurred())==NULL)
\t\t\tPyErr_SetString(#modulename#_error,\"Failure of a callback function\");
\t\tgoto capi_fail;
\t} else"""},
    """\tF_WRAPPEDFUNC(#name#,#NAME#)(#callfortran#);"""],
    '_check':isfunction_wrap,
    },{ # Function
    'docreturn':'#rname#,',
    'docstrout':'\t#pydocsignout#',
    'latexdocstrout':['\\item[]{{}\\verb@#pydocsignout#@{}}',
                      {hasresultnote:'--- #resultnote#'}],
    'callfortranroutine':[{l_and(debugcapi,isstringfunction):"""\
#ifdef USESCOMPAQFORTRAN
\tfprintf(stderr,\"debug-capi:Fortran function #ctype# #name#(#callcompaqfortran#)\\n\");
#else
\tfprintf(stderr,\"debug-capi:Fortran function #ctype# #name#(#callfortran#)\\n\");
#endif
"""},
                          {l_and(debugcapi,l_not(isstringfunction)):"""\
\tfprintf(stderr,\"debug-capi:Fortran function #ctype# #name#(#callfortran#)\\n\");
"""}
                          ],
    '_check':l_and(isfunction,l_not(isfunction_wrap))
    },{ # Scalar function
    'declfortranroutine':'extern #ctype# F_FUNC(#name#,#NAME#)();',
    'decl':[{iscomplexfunction_warn:'\t#ctype# #name#_return_value={0,0};',l_not(iscomplexfunction):'\t#ctype# #name#_return_value;'},
            {iscomplexfunction:'\tPyObject *#name#_return_value_capi = Py_None;'}],
    'callfortranroutine':[
        {hasexternals:"""\
\tif (#setjmpbuf#) {
\t\tif ((PyErr_Occurred())==NULL)
\t\t\tPyErr_SetString(#modulename#_error,\"Failure of a callback function\");
\t\tgoto capi_fail;
\t} else"""},
                          '\t#name#_return_value = F_FUNC(#name#,#NAME#)(#callfortran#);',
                          {l_and(debugcapi,iscomplexfunction):'\tfprintf(stderr,"#routdebugshowvalue#\\n",#name#_return_value.r,#name#_return_value.i);'},
                          {l_and(debugcapi,l_not(iscomplexfunction)):'\tfprintf(stderr,"#routdebugshowvalue#\\n",#name#_return_value);'}],
    'pyobjfrom':{iscomplexfunction:'\t#name#_return_value_capi = pyobj_from_#ctype#1(#name#_return_value);'},
    'need':['F_FUNC',{iscomplexfunction:'pyobj_from_#ctype#1'},{islong_longfunction:'long_long'},{islong_doublefunction:'long_double'}],
    'returnformat':'#rformat#',
    'return':{iscomplexfunction:',#name#_return_value_capi',
              l_not(iscomplexfunction):',#name#_return_value'},
    '_check':l_and(isfunction,l_not(isstringfunction),l_not(isfunction_wrap))
    },{ # String function
    'declfortranroutine':'extern void F_FUNC(#name#,#NAME#)();',
    'decl':['\t#ctype# #name#_return_value = NULL;',
            '\tint #name#_return_value_len;'],
    'callfortran':'#name#_return_value,#name#_return_value_len,',
    'callfortranroutine':['\t#name#_return_value_len = #rlength#;',
                          '\tSTRINGMALLOC(#name#_return_value,#name#_return_value_len);',
                          {hasexternals:"""\
\tif (#setjmpbuf#) {
\t\tif ((PyErr_Occurred())==NULL)
\t\t\tPyErr_SetString(#modulename#_error,\"Failure of a callback function\");
\t\tgoto capi_fail;
\t} else"""},
                          """\
#ifdef USESCOMPAQFORTRAN
\tF_FUNC(#name#,#NAME#)(#callcompaqfortran#);
#else
\tF_FUNC(#name#,#NAME#)(#callfortran#);
#endif
""",
                          {debugcapi:'\tfprintf(stderr,"#routdebugshowvalue#\\n",#name#_return_value_len,#name#_return_value);'}],
    'returnformat':'#rformat#',
    'return':',#name#_return_value',
    'freemem':'\tSTRINGFREE(#name#_return_value);',
    'need':['F_FUNC','#ctype#','STRINGMALLOC','STRINGFREE'],
    '_check':l_and(isstringfunction,l_not(isfunction_wrap))
    },{ # Debugging
    'routdebugenter':'\tfprintf(stderr,"debug-capi:Python C/API function #modulename#.#name#(#docsignature#)\\n");',
    'routdebugleave':'\tfprintf(stderr,"debug-capi:Python C/API function #modulename#.#name#: successful.\\n");',
    'routdebugfailure':'\tfprintf(stderr,"debug-capi:Python C/API function #modulename#.#name#: failure.\\n");',
    '_check':debugcapi
    }
    ]

################ Rules for arguments ##################

arg_rules=[
    {
    'separatorsfor':sepdict
    },
    { # Common
    'frompyobj':['\t/* Processing variable #varname# */',
                 {debugcapi:'\tfprintf(stderr,"#vardebuginfo#\\n");'},],
    '_depend':''
    },
# Doc signatures
    {
    'docstropt':{l_and(isoptional,isintent_nothide):'\t#pydocsign#'},
    'docstrreq':{l_and(isrequired,isintent_nothide):'\t#pydocsign#'},
    'docstrout':{isintent_out:'\t#pydocsignout#'},
    'latexdocstropt':{l_and(isoptional,isintent_nothide):['\\item[]{{}\\verb@#pydocsign#@{}}',
                                                          {hasnote:'--- #note#'}]},
    'latexdocstrreq':{l_and(isrequired,isintent_nothide):['\\item[]{{}\\verb@#pydocsign#@{}}',
                                                          {hasnote:'--- #note#'}]},
    'latexdocstrout':{isintent_out:['\\item[]{{}\\verb@#pydocsignout#@{}}',
                                    {l_and(hasnote,isintent_hide):'--- #note#',
                                     l_and(hasnote,isintent_nothide):'--- See above.'}]},
    'depend':''
    },
# Required/Optional arguments
    {
    'kwlist':'"#varname#",',
    'docsign':'#varname#,',
    '_check':l_and(isintent_nothide,l_not(isoptional))
    },
    {
    'kwlistopt':'"#varname#",',
    'docsignopt':'#varname#=#showinit#,',
    'docsignoptshort':'#varname#,',
    '_check':l_and(isintent_nothide,isoptional)
    },
# Docstring/BuildValue
    {
    'docreturn':'#varname#,',
    'returnformat':'#varrformat#',
    'need':{islong_long:'long_long',
            islong_double:'long_double',
            islong_complex:'complex_long_double',
            },
    '_check':isintent_out
    },
# Externals (call-back functions)
    { # Common
    'docsignxa':'#varname#_extra_args=(),',
    'docsignxashort':'#varname#_extra_args,',
    'docstropt':'\t#varname#_extra_args := () input tuple',
    'docstrcbs':'#cbdocstr#',
    'latexdocstrcbs':'\\item[] #cblatexdocstr#',
    'latexdocstropt':'\\item[]{{}\\verb@#varname#_extra_args := () input tuple@{}} --- Extra arguments for call-back function {{}\\verb@#varname#@{}}.',
    'decl':['\tPyObject *#varname#_capi = Py_None;',
            '\tPyTupleObject *#varname#_xa_capi = NULL;',
            '\tPyTupleObject *#varname#_args_capi = NULL;',
            '\tjmp_buf #varname#_jmpbuf;',
            '\tint #varname#_jmpbuf_flag = 0;',
            '\tint #varname#_nofargs_capi = 0;'
            ],
    'kwlistxa':'"#varname#_extra_args",',
    'argformat':{isrequired:'O!'},
    'keyformat':{isoptional:'O'},
    'xaformat':'O!',
    'args_capi':{isrequired:',&PyFunction_Type,&#varname#_capi'},
    'keys_capi':{isoptional:',&#varname#_capi'},
    'keys_xa':',&PyTuple_Type,&#varname#_xa_capi',
    'setjmpbuf':'(setjmp(#cbname#_jmpbuf))',
    'callfortran':'#cbname#,',
    'freemem':"""\
\tif (#varname#_jmpbuf_flag) {
\t\tCFUNCSMESS(\"Resetting jmpbuf for `#varname#`.\\n\");
\t\t#cbname#_capi = #varname#_capi;
\t\tPy_DECREF(#cbname#_args_capi);
\t\t#cbname#_args_capi = #varname#_args_capi;
\t\t#cbname#_nofargs = #varname#_nofargs_capi;
\t\tmemcpy(&#cbname#_jmpbuf,&#varname#_jmpbuf,sizeof(jmp_buf));
\t\t#varname#_jmpbuf_flag = 0;
\t}""",
    'need':['#cbname#','setjmp.h'],
    '_check':isexternal
    },{
    'frompyobj':["""\
\t#varname#_nofargs_capi = #cbname#_nofargs;
\tif (create_cb_arglist(#varname#_capi,#varname#_xa_capi,#maxnofargs#,#nofoptargs#,&#cbname#_nofargs,&#varname#_args_capi)) {
\t\tif ((PyErr_Occurred())==NULL)
\t\t\tPyErr_SetString(#modulename#_error,\"failed in processing argument list for call-back #varname#.\" );
\t\tgoto capi_fail;
\t}
\tSWAP(#varname#_capi,#cbname#_capi,PyObject);
\tSWAP(#varname#_args_capi,#cbname#_args_capi,PyTupleObject);
\tmemcpy(&#varname#_jmpbuf,&#cbname#_jmpbuf,sizeof(jmp_buf));
\t#varname#_jmpbuf_flag = 1;""",
                 {debugcapi:"""
\tfprintf(stderr,\"debug-capi:Assuming %d arguments; at most #maxnofargs#(-#nofoptargs#) is expected.\\n\",#cbname#_nofargs);
\tCFUNCSMESSPY(\"for #varname#=\",#cbname#_capi);
\tfprintf(stderr,\"#vardebugshowvalue# (call-back in C).\\n\",#cbname#);"""},
                 ],
    'need':['SWAP','create_cb_arglist'],
    '_check':isexternal,
    '_depend':''
    },
# Scalars (not complex)
    { # Common
    'decl':'\t#ctype# #varname#;',
    'pyobjfrom':{debugcapi:'\tfprintf(stderr,"#vardebugshowvalue#\\n",#varname#);'},
    'callfortran':'&#varname#,',
    'return':{isintent_out:',#varname#'},
    '_check':l_and(isscalar,l_not(iscomplex))
    },{
    'need':{hasinitvalue:'math.h'},
    '_check':l_and(isscalar,l_not(iscomplex)),
    '_depend':''
    },{ # Not hidden
    'decl':'\tPyObject *#varname#_capi = Py_None;',
    'argformat':{isrequired:'O'},
    'keyformat':{isoptional:'O'},
    'args_capi':{isrequired:',&#varname#_capi'},
    'keys_capi':{isoptional:',&#varname#_capi'},
    'pyobjfrom':{isintent_inout:'\tif (!try_pyarr_from_#ctype#(#varname#_capi,&#varname#)) goto capi_fail;'},
    'need':{isintent_inout:'try_pyarr_from_#ctype#'},
    '_check':l_and(isscalar,l_not(iscomplex),isintent_nothide)
    },{
    'frompyobj':[{hasinitvalue:'\tif (#varname#_capi == Py_None) #varname# = #init#; else'},
                 {l_and(isoptional,l_not(hasinitvalue)):'\tif (#varname#_capi != Py_None)'},
                 '\tif (!#ctype#_from_pyobj(&#varname#,#varname#_capi,"#ctype#_from_pyobj failed in converting #nth# `#varname#\' of #pyname# to C #ctype#\\n"))\n\t\tgoto capi_fail;'],
    'need':['#ctype#_from_pyobj'],
    '_check':l_and(isscalar,l_not(iscomplex),isintent_nothide),
    '_depend':''
    },{ # Hidden
    '_check':l_and(isscalar,l_not(iscomplex),isintent_hide)
    },{ # Hidden
    'frompyobj':{hasinitvalue:'\t#varname# = #init#;'},
    'need':{islong_long:'long_long',islong_double:'long_double'},
    '_check':l_and(isscalar,l_not(iscomplex),isintent_hide),
    '_depend':''
    },{ # Common
    'frompyobj':{debugcapi:'\tfprintf(stderr,"#vardebugshowvalue#\\n",#varname#);'},
    '_check':l_and(isscalar,l_not(iscomplex)),
    '_depend':''
    },
# Complex scalars
    { # Common
    'decl':'\t#ctype# #varname#;',
    'callfortran':'&#varname#,',
    'pyobjfrom':{debugcapi:'\tfprintf(stderr,"#vardebugshowvalue#\\n",#varname#.r,#varname#.i);'},
    'return':{isintent_out:',#varname#_capi'},
    '_check':iscomplex
    },{ # Not hidden
    'decl':'\tPyObject *#varname#_capi = Py_None;',
    'argformat':{isrequired:'O'},
    'keyformat':{isoptional:'O'},
    'args_capi':{isrequired:',&#varname#_capi'},
    'keys_capi':{isoptional:',&#varname#_capi'},
    'need':{isintent_inout:'try_pyarr_from_#ctype#'},
    'pyobjfrom':{isintent_inout:'\tif (!try_pyarr_from_#ctype#(#varname#_capi,&#varname#)) goto capi_fail;'},
    '_check':l_and(iscomplex,isintent_nothide)
    },{
    'frompyobj':[{hasinitvalue:'\tif (#varname#_capi==Py_None) {#varname#.r = #init.r#, #varname#.i = #init.i#;} else'},
                 {l_and(isoptional,l_not(hasinitvalue)):'\tif (#varname#_capi != Py_None)'},
                 '\tif (!#ctype#_from_pyobj(&#varname#,#varname#_capi,"#ctype#_from_pyobj failed in converting #nth# `#varname#\' of #pyname# to C #ctype#\\n"))\n\t\tgoto capi_fail;'],
    'need':['#ctype#_from_pyobj'],
    '_check':l_and(iscomplex,isintent_nothide),
    '_depend':''
    },{ # Hidden
    'decl':{isintent_out:'\tPyObject *#varname#_capi = Py_None;'},
    '_check':l_and(iscomplex,isintent_hide)
    },{
    'frompyobj': {hasinitvalue:'\t#varname#.r = #init.r#, #varname#.i = #init.i#;'},
    '_check':l_and(iscomplex,isintent_hide),
    '_depend':''
    },{ # Common
    'pyobjfrom':{isintent_out:'\t#varname#_capi = pyobj_from_#ctype#1(#varname#);'},
    'need':['pyobj_from_#ctype#1'],
    '_check':iscomplex
    },{
    'frompyobj':{debugcapi:'\tfprintf(stderr,"#vardebugshowvalue#\\n",#varname#.r,#varname#.i);'},
    '_check':iscomplex,
    '_depend':''
    },
# String
    { # Common
    'decl':['\t#ctype# #varname# = NULL;',
            '\tint slen(#varname#);',
            '\tPyObject *#varname#_capi = Py_None;'],
    'callfortran':'#varname#,',
    'callfortranappend':'slen(#varname#),',
    'pyobjfrom':{debugcapi:'\tfprintf(stderr,"#vardebugshowvalue#\\n",slen(#varname#),#varname#);'},
    'freemem':'\tSTRINGFREE(#varname#);',
    'return':{isintent_out:',#varname#'},
    'need':['len..','STRINGFREE'],
    '_check':isstring
    },{ # Common
    'frompyobj':['\tslen(#varname#) = #length#;',
                 """\
\tif (!#ctype#_from_pyobj(&#varname#,&slen(#varname#),#init#,#varname#_capi,\"#ctype#_from_pyobj failed in converting #nth# `#varname#\' of #pyname# to C #ctype#\"))
\t\tgoto capi_fail;"""],
    'need':['#ctype#_from_pyobj','len..'],
    '_check':isstring,
    '_depend':''
    },{ # Not hidden
    'argformat':{isrequired:'O'},
    'keyformat':{isoptional:'O'},
    'args_capi':{isrequired:',&#varname#_capi'},
    'keys_capi':{isoptional:',&#varname#_capi'},
    'pyobjfrom':{isintent_inout:'\tif (!try_pyarr_from_#ctype#(#varname#_capi,#varname#))\n\t\tgoto capi_fail;'},
    'need':{isintent_inout:'try_pyarr_from_#ctype#'},
    '_check':l_and(isstring,isintent_nothide)
    },{ # Hidden
    '_check':l_and(isstring,isintent_hide)
    },{
    'frompyobj':{debugcapi:'\tfprintf(stderr,"#vardebugshowvalue#\\n",slen(#varname#),#varname#);'},
    '_check':isstring,
    '_depend':''
    },
# Array
    { # Common
    'decl':['\t#ctype# *#varname# = NULL;',
            '\tint #varname#_Dims[#rank#] = {#rank*[-1]#};',
            '\tPyArrayObject *capi_#varname#_tmp = NULL;',
            ],
    'callfortran':'#varname#,',
    'return':{isintent_out:',capi_#varname#_tmp'},
    'need':'len..',
    '_check':isarray
    },{ 
    'frompyobj':'\t#setdims#;',
    'need':['arr_from_pyobj',{hasinitvalue:'forcomb'},{hasinitvalue:'CFUNCSMESS'}],
    '_check':isarray,
    '_depend':''
    },{ # Not hidden
    'decl':'\tPyObject *#varname#_capi = Py_None;',
    'argformat':{isrequired:'O'},
    'keyformat':{isoptional:'O'},
    'args_capi':{isrequired:',&#varname#_capi'},
    'keys_capi':{isoptional:',&#varname#_capi'},
    'pyobjfrom':{isintent_inout:"""\
\tif ((#varname#_capi != Py_None) && PyArray_Check(#varname#_capi) \\
\t\t&& (#varname#_capi != (PyObject *)capi_#varname#_tmp))
\t\tif (((PyArrayObject *)#varname#_capi)->nd != capi_#varname#_tmp->nd)
\t\t\tif (#varname#_capi != capi_#varname#_tmp->base)
\t\t\t\tcopy_ND_array((PyArrayObject *)capi_#varname#_tmp->base,(PyArrayObject *)#varname#_capi);
\t\t\telse ;
\t\telse
\t\t\tcopy_ND_array(capi_#varname#_tmp,(PyArrayObject *)#varname#_capi);"""},
    'need':{isintent_inout:'copy_ND_array'},
    'freemem':{l_not(isintent_out):"""\
\tif (capi_#varname#_tmp != NULL) Py_XDECREF(capi_#varname#_tmp->base);
\tPy_XDECREF(capi_#varname#_tmp);
"""},
    '_check':l_and(isarray,isintent_nothide)
    },{
    'frompyobj':[{isintent_hide:'\tcapi_#varname#_tmp = arr_from_pyobj(#atype#,#varname#_Dims,#rank#,Py_None);'},
                 {isintent_nothide:'\tcapi_#varname#_tmp = arr_from_pyobj(#atype#,#varname#_Dims,#rank#,#varname#_capi);'},
                 """\
\tif (capi_#varname#_tmp == NULL) {
\t\tPyErr_SetString(#modulename#_error,\"failed in converting #nth# `#varname#\' of #pyname# to C array\" );
\t\t\tgoto capi_fail;
\t}
\t#varname# = (#ctype# *)(capi_#varname#_tmp->data);""",
                  {hasinitvalue:[
                      {isintent_nothide:'\tif (#varname#_capi == Py_None) {'},
                      {isintent_hide:'\t{'},
                      {iscomplexarray:'\t\t#ctype# capi_c;'},
                      """\
\t\tint *_i,capi_i=0;
\t\tCFUNCSMESS(\"#name#: Initializing #varname#=#init#\\n\");
\t\tif (initforcomb(capi_#varname#_tmp->dimensions,capi_#varname#_tmp->nd,1))
\t\t\twhile (_i = nextforcomb())
\t\t\t\t#varname#[capi_i++] = #init#; /* fortran way */
\t\telse
\t\t\tPyErr_SetString(#modulename#_error,\"Initialization of #nth# #varname# failed.\");
\t}""",]}
                 ],
    '_check':isarray,
    '_depend':''
    },{ # Hidden
    'freemem':{l_not(isintent_out):'\tPy_DECREF(capi_#varname#_tmp);'},
    '_check':l_and(isarray,isintent_hide)
    },
# Scalararray
    { # Common
    '_check':l_and(isarray,l_not(iscomplexarray))
    },{ # Not hidden
    '_check':l_and(isarray,l_not(iscomplexarray),isintent_nothide)
    },
# Complexarray
    {'need':'#ctype#',
     '_check':iscomplexarray,
     '_depend':''
     }
# Stringarray
    ]

################# Rules for checking ###############

check_rules=[
    {
    'frompyobj':{debugcapi:'\tfprintf(stderr,\"debug-capi:Checking `#check#\'\\n\");'},
    'need':'len..'
    },{
    'frompyobj':'\tCHECKSCALAR(#check#,\"#check#\",\"#nth# #varname#\",\"#varshowvalue#\",#varname#);',
    'need':'CHECKSCALAR',
    '_check':l_and(isscalar,l_not(iscomplex)),
    '_break':''
    },{
    'frompyobj':'\tCHECKSTRING(#check#,\"#check#\",\"#nth# #varname#\",\"#varshowvalue#\",#varname#);',
    'need':'CHECKSTRING',
    '_check':isstring,
    '_break':''
    },{
    'need':'CHECKARRAY',
    'frompyobj':'\tCHECKARRAY(#check#,\"#check#\",\"#nth# #varname#\");',
    '_check':isarray,
    '_break':''
    },{
    'need':'CHECKGENERIC',
    'frompyobj':'\tCHECKGENERIC(#check#,\"#check#\",\"#nth# #varname#\");'}
]

########## Applying the rules. No need to modify what follows #############

#################### Build C/API module #######################

def buildmodule(m,um):
    global f2py_version,options
    outmess('\tBuilding module "%s"...\n'%(m['name']))
    mod_rules=defmod_rules[:]
    vrd=modsign2map(m)
    rd=dictappend({'f2py_version':f2py_version},vrd)
    funcwrappers = []
    for n in m['interfaced']:
        nb=None
        for bi in m['body']:
            if not bi['block']=='interface':
                errmess('buildmodule: Expected interface block. Skipping.\n')
                continue
            for b in bi['body']:
                if b['name']==n: nb=b;break
                    
        if not nb:
            errmess('buildmodule: Could not found the body of interfaced routine "%s". Skipping.\n'%(n))
            continue
        api,wrap=buildapi(nb)
        if wrap:
            funcwrappers.append(wrap)
        ar=applyrules(api,vrd)
        rd=dictappend(rd,ar)


    # Construct F90 module support
    mr,wrap = f90mod_rules.buildhooks(m)
    if wrap:
        funcwrappers.append(wrap)
    ar=applyrules(mr,vrd)
    rd=dictappend(rd,ar)

    for bi in m['body']:
        if isroutine(bi) and bi['name'] not in m['interfaced']:
            bi['modulename'] = m['name'] # is module routine
            api,wrap=buildapi(bi)
            if wrap:
                funcwrappers.append(wrap)
            ar=applyrules(api,vrd)
            rd=dictappend(rd,ar)

    for u in um:
        ar=use_rules.buildusevars(u,m['use'][u['name']])
        rd=dictappend(rd,ar)

    needs=cfuncs.get_needs()
    code={}
    for n in needs.keys():
        code[n]=[]
        for k in needs[n]:
            c=''
            if cfuncs.includes0.has_key(k): c=cfuncs.includes0[k]
            elif cfuncs.includes.has_key(k): c=cfuncs.includes[k]
            elif cfuncs.userincludes.has_key(k): c=cfuncs.userincludes[k]
            elif cfuncs.typedefs.has_key(k): c=cfuncs.typedefs[k]
            elif cfuncs.cppmacros.has_key(k): c=cfuncs.cppmacros[k]
            elif cfuncs.cfuncs.has_key(k): c=cfuncs.cfuncs[k]
            elif cfuncs.callbacks.has_key(k): c=cfuncs.callbacks[k]
            elif cfuncs.f90modhooks.has_key(k): c=cfuncs.f90modhooks[k]
            else: errmess('buildmodule: unknown need %s.\n'%(`k`));continue
            code[n].append(c)
    mod_rules.append(code)
    for r in mod_rules:
        if (r.has_key('_check') and r['_check'](m)) or (not r.has_key('_check')):
            ar=applyrules(r,vrd,m)
            rd=dictappend(rd,ar)
    ar=applyrules(module_rules,rd)
    f=open(vrd['modulename']+'module.c','w')
    f.write(string.replace(ar['modulebody'],'\t',2*' '))
    f.close()
    outmess('\tWrote C/API module "%s" to file "%smodule.c"\n'%(m['name'],vrd['modulename']))
    f=open(vrd['modulename']+'module.tex','w')
    f.write('%% This file is auto-generated with f2py (version:%s)\n'%(f2py_version))
    if not options.has_key('shortlatex'):
        f.write('\\documentclass{article}\n\\usepackage{a4wide}\n\\begin{document}\n\\tableofcontents\n\n')
    f.write(string.join(ar['latexdoc'],'\n'))
    if not options.has_key('shortlatex'):
        f.write('\\end{document}')
    f.close()
    outmess('\tDocumentation is saved to file "%smodule.tex"\n'%(vrd['modulename']))
    if funcwrappers:
        wn = '%s-f2pywrappers.f'%(vrd['modulename'])
        f=open(wn,'w')
        f.write('C     This file is autogenerated with f2py (version:%s)\n'%(f2py_version))
        f.write('C     It contains Fortran wrappers to fortran functions.\n')
        f.write(string.join(funcwrappers,'\n\n')+'\n')
        f.close()
        outmess('\tFortran wrappers are saved to "%s"\n'%(wn))

################## Build C/API function #############

stnd={1:'st',2:'nd',3:'rd',4:'th',5:'th',6:'th',7:'th',8:'th',9:'th',0:'th'}
def buildapi(rout):
    rout,wrap = func2subr.assubr(rout)
    args,depargs=getargs(rout)
    capi_maps.depargs=depargs
    var=rout['vars']
    outmess('\t\tConstructing wrapper function "%s"...\n'%(rout['name']))
    # Routine
    vrd=routsign2map(rout)
    rd=dictappend({},vrd)
    for r in rout_rules:
        if (r.has_key('_check') and r['_check'](rout)) or (not r.has_key('_check')):
            ar=applyrules(r,vrd,rout)
            rd=dictappend(rd,ar)
    # Args
    nth,nthk=0,0
    savevrd={}
    for a in args:
        vrd=sign2map(a,var[a])
        if not isintent_hide(var[a]):
            if not isoptional(var[a]):
                nth=nth+1
                vrd['nth']=`nth`+stnd[nth%10]+' argument'
            else:
                nthk=nthk+1
                vrd['nth']=`nthk`+stnd[nthk%10]+' keyword'
        else: vrd['nth']='hidden'
        savevrd[a]=vrd
        for r in arg_rules:
            if r.has_key('_depend'): continue
            if (r.has_key('_check') and r['_check'](var[a])) or (not r.has_key('_check')):
                ar=applyrules(r,vrd,var[a])
                rd=dictappend(rd,ar)
                if r.has_key('_break'): break
    for a in depargs:
        vrd=savevrd[a]
        for r in arg_rules:
            if not r.has_key('_depend'): continue
            if (r.has_key('_check') and r['_check'](var[a])) or (not r.has_key('_check')):
                ar=applyrules(r,vrd,var[a])
                rd=dictappend(rd,ar)
                if r.has_key('_break'): break
        if var[a].has_key('check'):
            for c in var[a]['check']:
                vrd['check']=c
                ar=applyrules(check_rules,vrd,var[a])
                rd=dictappend(rd,ar)
    rd['docsignature']=stripcomma(replace('#docsign##docsignopt##docsignxa#',
                                          {'docsign':rd['docsign'],
                                           'docsignopt':rd['docsignopt'],
                                           'docsignxa':rd['docsignxa']}))
    optargs=stripcomma(replace('#docsignopt##docsignxa#',
                               {'docsignxa':rd['docsignxashort'],
                                'docsignopt':rd['docsignoptshort']}
                               ))
    if optargs=='':
        rd['docsignatureshort']=stripcomma(replace('#docsign#',{'docsign':rd['docsign']}))
    else:
        rd['docsignatureshort']=replace('#docsign#[#docsignopt#]',
                                        {'docsign':rd['docsign'],
                                         'docsignopt':optargs,
                                         })
    rd['latexdocsignatureshort']=string.replace(rd['docsignatureshort'],'_','\\_')
    rd['latexdocsignatureshort']=string.replace(rd['latexdocsignatureshort'],',',', ')
    cfs=stripcomma(replace('#callfortran##callfortranappend#',{'callfortran':rd['callfortran'],'callfortranappend':rd['callfortranappend']}))
    if len(rd['callfortranappend'])>1:
        rd['callcompaqfortran']=stripcomma(replace('#callfortran# 0,#callfortranappend#',{'callfortran':rd['callfortran'],'callfortranappend':rd['callfortranappend']}))
    else:
        rd['callcompaqfortran']=cfs
    rd['callfortran']=cfs
    if type(rd['docreturn'])==types.ListType:
        rd['docreturn']=stripcomma(replace('#docreturn#',{'docreturn':rd['docreturn']}))+' = '
    rd['docstrsigns']=[]
    rd['latexdocstrsigns']=[]
    for k in ['docstrreq','docstropt','docstrout','docstrcbs']:
        if rd.has_key(k) and type(rd[k])==types.ListType:
            rd['docstrsigns']=rd['docstrsigns']+rd[k]
        k='latex'+k
        if rd.has_key(k) and type(rd[k])==types.ListType:
            rd['latexdocstrsigns']=rd['latexdocstrsigns']+rd[k][0:1]+\
                                    ['\\begin{description}']+rd[k][1:]+\
                                    ['\\end{description}']
    ar=applyrules(routine_rules,rd)
    print '\t\t  %s'%ar['docshort']
    if hascommon(rout):
        cr=common_rules.buildcommonblocks(rout)
        ar=dictappend(ar,cr)
    return ar,wrap



#################### EOF rules.py #######################

